@@ -13,6 +13,7 @@ def partial_list_repr(lst, max_len):
             ', '.join(repr(item) for item in lst[:max_len]),
             len(lst))
 
+
 class Blend(LabeledView):
     def __init__(self, tensors, weights=None, factor=None, k_values=1, svals=None, auto_build_tensor=True):
         '''
@@ -53,7 +54,7 @@ class Blend(LabeledView):
         self.auto_build_tensor = auto_build_tensor
 
     def __repr__(self):
-        return u'<Blend of %s, weights=%s>' % (partial_list_repr(self.tensors, 2), partial_list_repr(self.weights, 2))
+        return u'<Blend of %s, weights=%s>' % (partial_list_repr(self.names, 3), partial_list_repr(self.weights, 3))
 
     def __getstate__(self):
         return dict(
@@ -75,17 +76,35 @@ class Blend(LabeledView):
         self._svals = state.get('svals', None)
         self.weights = state['weights']
         self.auto_build_tensor = state.get('auto_build_tensor', True)
-        
+
     def bake(self):
         '''
         Return a normal LabeledView with the current contents of the blend.
         '''
         if self._tensor is None: self.build_tensor()
         return LabeledView(self.tensor, self._labels)
-    
+
     def _set_tensors(self, tensors):
+        '''
+        Set the input tensors. Computes the label lists also. You
+        should not call this function directly; rather, assign to
+        blend.tensors.
+
+        You can pass a ``dict`` or sequence of ``(label, tensor)``
+        pairs; the tensors will be labeled according to the keys.
+        '''
+        if hasattr(tensors, 'items'):
+            # Extract the items, if we have some.
+            tensors = tensors.items()
+        if isinstance(tensors[0], (list, tuple)):
+            # Assign names. Don't call `dict()`, in case a sequence
+            # was passed and two tensors have the same label.
+            names, tensors = zip(*tensors)
+        else:
+            names = map(repr, tensors)
         self._tensors = tuple(tensors)
-        self.logger.info('tensors=%r', self._tensors)
+        self.names = tuple(names)
+        self.logger.info('tensors: %s', ', '.join(self.names))
         self.ndim = ndim = tensors[0].ndim
         if not all(tensor.ndim == ndim for tensor in tensors):
             raise TypeError('Blended tensors must have the same dimensionality.')
@@ -106,7 +125,7 @@ class Blend(LabeledView):
         self.logger.info('Done making ordered sets. label_overlap: %r', label_overlap)
         if not any(label_overlap):
             self.logger.warn('No labels overlap.')
-        
+
         # Invalidate other data
         self._weights = self._tensor = self._svals = None
 
@@ -121,10 +140,10 @@ class Blend(LabeledView):
         '''
         if self._svals is None: self._svals = [[]]*len(self._tensors)
         if num_svals > len(self._svals[tensor_idx] or []):
-            self.logger.info('computing by-tensor SVD for tensor %d, k=%d', tensor_idx, num_svals)
+            self.logger.info('computing SVD(k=%d) for %s', num_svals, self.names[tensor_idx])
             self._svals[tensor_idx] = self._tensors[tensor_idx].svd(k=num_svals).svals.values()
         return self._svals[tensor_idx][:num_svals]
-    
+
     def rough_weight(self, tensor_idx):
         '''
         Compute the rough weight for one of the input tensors.
@@ -132,12 +151,19 @@ class Blend(LabeledView):
         k = self.k_values
         if isinstance(k, (list, tuple)): k = k[tensor_idx]
         return 1.0/sqrt(sum([x*x for x in self.tensor_svals(tensor_idx, k)[:k]]))
-    
+
     def _set_weights(self, weights):
         if weights is None:
             # Rough blend
             self._weights = [self.rough_weight(tensor) for tensor in xrange(len(self.tensors))]
             self.normalize_weights()
+        elif weights == '=':
+            # Equal weights, summing to 1
+            self._weights = [1]*len(self.tensors)
+            self.normalize_weights()
+        elif isinstance(weights, (int, long, float)):
+            # Equal weights of the given value
+            self._weights = [float(weights)]*len(self.tensors)
         else:
             # Explicit
             if weights == self._weights: return # If same, no-op.
@@ -159,7 +185,7 @@ class Blend(LabeledView):
             raise ValueError('factor must be between 0 and 1.')
         self.weights = [1.0-factor, float(factor)]
     factor = property(_get_factor, _set_factor)
-    
+
     def normalize_weights(self):
         '''
         Make the weights sum to 1.
@@ -194,8 +220,8 @@ class Blend(LabeledView):
             tensor.update((tuple(label_list.index(label) for label_list, label in izip(labels, key)), val)
                           for key, val in self._fast_iteritems())
         else:
-            for factor, cur_tensor in zip(self._weights, self._tensors):
-                self.logger.info('slow-merging %r' % cur_tensor)
+            for factor, cur_tensor, name in zip(self._weights, self._tensors, self.names):
+                self.logger.info('slow-merging %s' % name)
                 for key, val in cur_tensor.iteritems():
                     tensor.inc(tuple(label_list.index(label) for label_list, label in izip(labels, key)), factor*val)
         self._tensor = tensor
@@ -261,14 +287,14 @@ class Blend(LabeledView):
         return ((key, factor*val)
                 for factor, cur_tensor in zip(self._weights, self._tensors)
                 for key, val in cur_tensor.iteritems())
-    
+
     def iteritems(self):
         if self._keys_never_overlap:
             return self._fast_iteritems()
         else:
             return super(Blend, self).iteritems()
 
-        
+
     def __len__(self):
         if self._keys_never_overlap:
             return sum(map(len, self.tensors))
@@ -276,66 +302,80 @@ class Blend(LabeledView):
             return len(self.tensor)
 
     # Visualization
-    def coverage_plot(self, bin_size=50):
+    def coverage(self, bin_size=50):
         '''
-        Return a NumPy integer array specifying where each item in the tensor comes from.
+        Compute the coverage of the blend space by the input tensors.
 
-        Blank cells are 0; cells that come from tensor n have value
-        n+1. (If multiple tensors write in the same bin, the last one
-        wins.)
+        Returns NumPy 2D arrays ``(fill, magnitude, src)``. ``fill``
+        indicates how densely filled each "bin" is, from 0.0 (empty)
+        to 1.0 (full). ``magnitude`` accumulates the absolute values
+        of the items within the bin. ``src`` indicates which tensor
+        each item comes from, specified by its index in the
+        ``tensors`` array.  (If multiple tensors write in the same
+        bin, the last one wins.)
         '''
         if not isinstance(bin_size, (list, tuple)):
             bin_size = [bin_size]*self.ndim
-            
+
         import numpy
         src = numpy.zeros(tuple(numpy.ceil(float(items) / float(bins))
                                 for items, bins in izip(self.shape, bin_size)),
-                          dtype=numpy.int8)
+                          dtype=numpy.uint8)
+        magnitude = numpy.zeros(src.shape)
         fill = numpy.zeros(src.shape)
-        inc = 1.0 / numpy.product(src.shape)
-        
+        inc = 1.0 / numpy.product(bin_size)
+
         # This loop should look a lot like the one in FakeTensor.
         labels = self._labels
         for tensor_idx, tensor in enumerate(self._tensors):
             for key, val in tensor.iteritems():
-                idx = tuple(label_list.index(label) / bins for label_list, label, bins in izip(labels, key, bin_size))
-                src[idx] = tensor_idx + 1
+                idx = tuple(label_list.index(label) // bins for label_list, label, bins in izip(labels, key, bin_size))
+                src[idx] = tensor_idx
                 fill[idx] += inc
-        return fill, src
+                magnitude[idx] += abs(val)
+        return fill, magnitude, src
 
-    def coverage_pdf(self, canvas, bin_size=50, origin=None, inches_per_bin=None):
-        from reportlab.lib.units import inch
-        from reportlab.lib.pagesizes import letter
-        from colorsys import hsv_to_rgb
+    def coverage_image(self, width=None, height=None, pixel_size=None, *a, **kw):
+        '''
+        Generate a coverage image of this blend. You can specify the size of the image in one of two ways:
 
-        scratch = False
-        if isinstance(canvas, basestring):
-            from reportlab.pdfgen import canvas as pdfgen_canvas
-            canvas = pdfgen_canvas.Canvas(canvas, pagesize=letter)
-            scratch = True
-            
-        if origin is None: origin = [0.5*inch]*2
-        if inches_per_bin is None:
-            inches_per_bin = 0.1 # TODO: set dynamically
+        ``pixel_size``: the size of a pixel in rows and columns
+          (defaults to square if a single number is passed)
 
-        import numpy
-        hues = numpy.linspace(0, 1, len(self.tensors)+1)
-        
-        fill, src = self.coverage_plot(bin_size)
-        rows, cols = src.shape
-        bin_phys_size = inches_per_bin*inch
-        for row in xrange(rows):
-            for col in xrange(cols):
-                source = src[row,col]
-                if not source: continue
-                canvas.setStrokeColorRGB(*hsv_to_rgb(hues[source], 1, 1-fill[row, col]))
-                canvas.rect(origin[0] + bin_phys_size*col, origin[1] + bin_phys_size*row,
-                            bin_phys_size, bin_phys_size, fill=True)
+        ``width`` and/or ``height``: the target width and height of
+          the image. If it doesn't fit evenly, the image may be
+          slightly bigger than you specify. Defaults to square if one
+          or the other is unspecified.
 
-        if scratch:
-            canvas.showPage()
-            canvas.save()
+        Or if you give no parameters, the width defaults to 1000 pixels.
 
+        For more information, see ``csc.divisi.blend.CoverageImage``.
+        '''
+        # Compute the image size.
+        if pixel_size is None and width is None and height is None:
+            # Default to 1000 pixels wide.
+            width = 1000
+
+        if pixel_size is None:
+            # Compute the dimensions that are specified.
+            pixel_width = pixel_height = None
+            if width is not None:
+                pixel_width = self.shape[1] // width
+            if height is not None:
+                pixel_height = self.shape[0] // height
+
+            # Fill in, defaulting to square.
+            pixel_size = (pixel_width  if pixel_width  is not None else pixel_height,
+                          pixel_height if pixel_height is not None else pixel_width)
+
+        else:
+            if width is not None or height is not None:
+                raise TypeError("Can't specify both pixel_size and width/height.")
+
+        # Generate the raw coverage data.
+        fill, magnitude, src = self.coverage(pixel_size)
+
+        return CoverageImage(fill, magnitude, src, self.names, *a, **kw)
 
     # Blend analysis utilities
     def predicted_svals(self, num=50, for_each_tensor=None, track_origin=False):
@@ -365,7 +405,7 @@ class Blend(LabeledView):
                  for sval in self.tensor_svals(idx, for_each_tensor)]
         svals.sort(reverse=True)
         return svals[:num]
-        
+
     def total_veering(self, num=50, for_each_tensor=None, actual_svals=None):
         '''
         Calculate total veering.
@@ -373,7 +413,7 @@ class Blend(LabeledView):
         If you already have the singular values, pass them in as a list / array
         for a faster result.
         '''
-        
+
         predicted_svals = self.predicted_svals(num, for_each_tensor)
         if actual_svals is None:
             self.logger.info('computing actual singular values')
@@ -396,7 +436,7 @@ class Blend(LabeledView):
         # FIXME: take advantage of the fact that the labels don't change.
         return Blend(self.tensors, factor=factor,
                      k_values=self.k_values, svals=self._svals)
-        
+
     def compressed_svd_u(self, k=100):
         """
         Not done yet. --Rob
@@ -406,3 +446,110 @@ class Blend(LabeledView):
             labelset += set(t.label_list(0))
         ulabels = OrderedSet(list(labelset))
         svds = [t.svd(k) for t in self.weights]
+
+
+class CoverageImage(object):
+    def __init__(self, fill, magnitude, src, names):
+        '''
+        Create a coverage image.
+
+        Each input gets a color. Intensity indicates density.
+
+        If you have PIL, you can call ``.save(filename)`` on the
+        resulting object to save an image. Otherwise, there will be a
+        plain NumPy array at ``.arr``.
+        '''
+        from colorsys import hsv_to_rgb
+        import numpy
+
+        # Create the hues array.
+        n = len(names)
+        hues = numpy.linspace(0, 1, n+1)
+
+        # Re-order the hues for the maximum separation between adjacent items.
+        increment = (n+1)/2.0
+        hues = [hues[int(i*increment) % n] for i in xrange(n)]
+
+        rows, cols = src.shape
+
+        # Scale "fill" values to the dynamic range.
+        fill_scale = self.fill_scale = 1.0/fill.max()
+
+        ## Scale "magnitude" values to the dynamic range. Avoid having
+        ## too low saturation, so we set a minimum.
+        #min_saturation = 0.5
+        #magnitude_scale = self.magnitude_scale = (1.0-min_saturation)/magnitude.max()
+
+        # Create an empty white image.
+        img = numpy.zeros((rows, cols, 3), dtype=numpy.uint8)
+        img[:,:,:] = 255
+
+        # Fill it.
+        for row in xrange(rows):
+            for col in xrange(cols):
+                idx = row, col
+                fill_amt = fill[idx]
+                if not fill_amt: continue # skip if empty.
+                rgb = numpy.array(hsv_to_rgb(hues[src[idx]],
+                                             1,#magnitude[idx] * magnitude_scale + min_saturation,
+                                             1-fill_amt*fill_scale))
+                img[row, col, :] = rgb*255
+
+        self.names = names
+        self.arr = img
+        self.hues = hues
+
+    @property
+    def img(self):
+        import Image
+        return Image.fromarray(self.arr)
+
+    def save(self, filename, *a, **kw):
+        return self.img.save(filename, *a, **kw)
+
+    @property
+    def colors(self):
+        from colorsys import hsv_to_rgb
+        return [hsv_to_rgb(hue, 1, 1) for hue in self.hues]
+
+
+    def save_pdf(self, filename, margins=(1,1)):
+        '''
+        Make and save a PDF of this coverage plot, including a legend.
+
+        Margins are expressed in inches: (top-bottom, left-right).
+        '''
+        from reportlab.lib.units import inch
+        from reportlab.lib.pagesizes import letter
+        from reportlab.pdfgen.canvas import Canvas
+
+        c = Canvas(filename, pagesize=letter)
+
+        # Compute margins.
+        margin_top, margin_left = margins
+        margin_top *= inch; margin_left *= inch
+        whole_page_width, whole_page_height = letter
+        page_top = whole_page_height - margin_top
+        page_left = margin_left
+        page_width = whole_page_width - 2*margin_left
+
+        # Show the main image.
+        image = self.img
+        image_width = page_width
+        image_height = image_width / image.size[0] * image.size[1]
+        image_x = page_left
+        image_y = page_top - image_height
+        c.drawInlineImage(image, image_x, image_y, width=image_width, height=image_height)
+
+        # Draw legends beneath the image.
+        textobject = c.beginText()
+        textobject.setTextOrigin(page_left, image_y - .5*inch)
+        textobject.setFont('Helvetica', 14)
+        for name, color in izip(self.names, self.colors):
+            textobject.setFillColorRGB(*color)
+            textobject.textLine(name)
+        c.drawText(textobject)
+
+        # Done.
+        c.showPage()
+        c.save()
