@@ -6,8 +6,31 @@ from operator import itemgetter
 from itertools import izip
 
 # Some convenience functions
+
+def make_sparse_labeled_tensor(ndim, labels=None,
+                               initial=None, accumulate=None,
+                               normalize=False):
+    '''
+    Create a sparse labeled tensor.
+
+    ndim: number of dimensions (usually 2)
     
-def make_sparse_labeled_tensor(ndim, labels=None, initial=None, accumulate=None):
+    labels: if you already have label lists, pass them in here. (A
+    None in this list means an unlabeled dimension. If you simply
+    don't have labels yet, pass an OrderedSet().)
+
+    initial / accumulate: sequences of (key, value) pairs to add to
+    the tensor. ``initial`` is applied first by ``.update``, meaning
+    that later values will override earlier ones. ``accumulate`` is
+    applied afterwards, and all values add to anything already there.
+
+    normalize:
+     an int or tuple of ints: normalize along that dimension
+     True: normalize along axis 0
+     'tfidf': use tf-idf
+     'tfidf.T': use tf-idf, transposed (matrix is documents by terms)
+     a class: adds that class as a layer.
+    '''
     from csc.divisi.tensor import DictTensor
     if labels is None: labels = [OrderedSet() for _ in xrange(ndim)]
     tensor = LabeledView(DictTensor(ndim), labels)
@@ -15,7 +38,11 @@ def make_sparse_labeled_tensor(ndim, labels=None, initial=None, accumulate=None)
         tensor.update(initial)
     for k, v in accumulate or []:
         tensor.inc(k, v)
-    return tensor
+
+    if normalize:
+        return tensor.normalized(normalize)
+    else:
+        return tensor
 
 
 def make_dense_labeled_tensor(data, labels):
@@ -37,11 +64,11 @@ class LabelingMixin(object):
         """
         Get the label for a particular place in the matrix -- that is,
         a given index along a given mode.
-        
+
         The inverse of this function is index().
         """
         return self._labels[mode][index]
-    
+
     def row_label(self, index): return self.label(0, index)
     def column_label(self, index): return self.label(1, index)
     def labels(self, indices):
@@ -62,7 +89,7 @@ class LabelingMixin(object):
     def index(self, mode, label, create=False):
         """
         Find the index, along a given mode, that is labeled with a given label.
-        
+
         The inverse of this function is label().
         """
         if label == TheEmptySlice: return TheEmptySlice
@@ -104,7 +131,7 @@ class LabeledView(LabelingMixin, View):
             label_lists = [None]*self.tensor.ndim
 
         if tensor.ndim != len(label_lists):
-            raise IndexError('Number of label lists (%d) does not match number of dimensions (%s)', (len(label_lists), tensor.ndim))
+            raise IndexError('Number of label lists (%d) does not match number of dimensions (%s)' % (len(label_lists), tensor.ndim))
 
         self._labels = [indexable_set(l, shape[mode])
                         for mode, l in enumerate(label_lists)]
@@ -136,24 +163,29 @@ class LabeledView(LabelingMixin, View):
                 return False
         return True
 
-    
+
     def transpose(self):
         if self.tensor.ndim != 2:
             raise TypeError('Only 2D tensors can be transposed.')
         return LabeledView(self.tensor.transpose(), [self._labels[1], self._labels[0]])
-    
+
     @property
     def T(self):
         return self.transpose()
 
     def __iter__(self):
-        return (self.labels(idx) for idx in self.tensor.__iter__())
+        labels = self._labels
+        for indices, v in self.tensor.iteritems():
+            label = tuple(label_list[index] for label_list, index in izip(labels, indices))
+            if None in label: continue
+            yield label
 
     def iteritems(self):
         labels = self._labels
         for indices, v in self.tensor.iteritems():
-            yield (tuple(label_list[index] for label_list, index in izip(labels, indices)),
-                   v)
+            result = (tuple(label_list[index] for label_list, index in izip(labels, indices)), v)
+            if None in result[0]: continue
+            yield result
 
     def iter_dim_keys(self, dim):
         return iter(self.label_list(dim))
@@ -184,7 +216,7 @@ class LabeledView(LabelingMixin, View):
             return False
 
         return indices in self.tensor
-        
+
     def __getitem__(self, labels):
         # Handle single dimensions
         if not isinstance(labels, (list, tuple)):
@@ -220,7 +252,7 @@ class LabeledView(LabelingMixin, View):
         # Make sure we have ordered numbers for each label
         for mode in xrange(d):
             self._labels[mode].add(labels[mode])
-        
+
         self.tensor[self.indices(labels)] = value
         # TODO: Add support for slice assignment.
 
@@ -252,7 +284,7 @@ class LabeledView(LabelingMixin, View):
             if v:
                 self[k] = self.get(k, 0) + v
         return self
-        
+
     def __neg__(self):
         return self.layer_on(-self.tensor)
 
@@ -323,7 +355,7 @@ class LabeledView(LabelingMixin, View):
             _svd = self.tensor.incremental_svd(*a, **kw)
             return LabeledSVD2DResults.layer_on(_svd, self)
         else:
-            raise NotImplementedError("Incremental SVDs can only be run on 2d tensors!")        
+            raise NotImplementedError("Incremental SVDs can only be run on 2d tensors!")
 
     ### Views
     def layer_on(self, tensor):
@@ -334,20 +366,20 @@ class LabeledView(LabelingMixin, View):
         new_tensor = self.tensor.zero_mean_normalized(mode=mode)
 
         new_labels = copy.deepcopy(self._labels)
-        # Duplicate labels, preserving the property that 
+        # Duplicate labels, preserving the property that
         # index(label)[mode] = index(prefix  label)[mode] - self.tensor.shape[mode]
         for i in xrange(self.tensor.shape[mode]):
             l = self.label(mode, i)
             new_labels[mode].add(prefix + l)
 
         return LabeledView(new_tensor, new_labels)
-        
-    
+
+
     def array_op(self, op, labels=None, *args, **kwargs):
         """
         Apply an operation to the tensor inside (which must be a
         DenseTensor), and return the result wrapped in the same view.
-        
+
         This assumes the resulting tensor has the same shape, unless
         you provide a labels= parameter.
         """
@@ -355,7 +387,7 @@ class LabeledView(LabelingMixin, View):
         if labels is None: labels = self._labels
         return inner.labeled(labels)
 
-    
+
     ### Utility extractions
     def vector_to_dict(self):
         '''Like #items for 1D tensors, but uses indices directly instead of
@@ -383,7 +415,7 @@ class LabeledView(LabelingMixin, View):
             warnings.warn(DeprecationWarning('abs_val is deprecated. use key=abs.'))
             assert key is None
             key = abs
-            
+
         if key is not None:
             extreme_key = lambda ent: key(ent[1])
         else:
@@ -397,6 +429,11 @@ class LabeledView(LabelingMixin, View):
         else:
             return [(k, float(v)) for k, v in items]
 
+    def bake(self):
+        '''
+        Simplify the representation.
+        '''
+        return make_sparse_labeled_tensor(ndim=self.ndim, initial=self.iteritems())
 
     def to_dense(self):
         '''Change the underlying representation to a dense tensor.'''
@@ -416,12 +453,19 @@ class LabeledView(LabelingMixin, View):
         '''
         return make_sparse_labeled_tensor(ndim=self.ndim, initial=(
                 (k, v) for k, v in self.iteritems() if v))
-    
+
     def k_means(self, k=20, **kw):
         means, clusters = self.tensor.k_means(k, **kw)
-        labeled_clusters = [self.label(0, index) for index in cluster]
+        labeled_clusters = [[self.label(0, index) for index in cluster] for cluster in clusters]
         return means, labeled_clusters
 
+    def label_histograms(self):
+        counts, magnitudes = self.tensor.label_histograms()
+        return ([LabeledView(count, [labels]) for count, labels
+                 in izip(counts, self.label_sets())],
+                [LabeledView(mag, [labels]) for mag, labels
+                 in izip(magnitudes, self.label_sets())])
+
 
 from csc.divisi.svd import SVD2DResults
 class LabeledSVD2DResults(SVD2DResults):
@@ -444,14 +488,14 @@ class LabeledSVD2DResults(SVD2DResults):
             def store_tensor(name, tensor):
                 data = tensor._data
                 arr = fileh.createCArray(root, name, Atom.from_dtype(data.dtype), tensor.shape, filters=filters)
-                arr[:] = data                
+                arr[:] = data
             # Labeled stuff
             for name in ('u', 'v', 'weighted_u', 'weighted_v'):
                 store_tensor(name, getattr(self, name).tensor)
             # Unlabeled stuff
             for name in ('svals', 'core'):
                 store_tensor(name, getattr(self, name))
-                
+
             # Ordered sets
             def write_labels(name, view):
                 arr = fileh.createVLArray(root, name, ObjectAtom(), filters=filters)
@@ -461,7 +505,7 @@ class LabeledSVD2DResults(SVD2DResults):
             write_labels('v_labels', self.v)
         finally:
             fileh.close()
-        
+
     @classmethod
     def load_pytables(cls, filename, copy=False):
         import numpy as np
