@@ -63,14 +63,14 @@ class ReconstructedTensor(Tensor):
                 a = a.tensordot(smoothing, i)
             self._cache[key] = a
         return self._cache[key]
-        
+
 
 class SVDResults(object):
     """
     Stores the results of an arbitrary higher-order SVD. If you are simply
     taking the SVD of a matrix, you are probably looking for the SVD2DResults
     class.
-    
+
     self.r is a list of SVD2DResults, one for each unfolding of the tensor.
     """
     def __init__(self, orig, sub_results):
@@ -120,7 +120,7 @@ class SVD2DResults(object):
 
     def __repr__(self):
         return u'<SVD2DResults: %d by %d, rank=%d>' % (self.u.shape[0], self.v.shape[0], len(self.svals))
-        
+
     def clear_cache(self):
         '''
         Delete all cached tensors from this object.
@@ -136,7 +136,7 @@ class SVD2DResults(object):
         if self._core is None:
             self._core = DenseTensor(numpy.diag(self.svals.to_dense()._data))
         return self._core
-    
+
     @property
     def weighted_u(self):
         '''
@@ -146,7 +146,7 @@ class SVD2DResults(object):
         if self._weighted_u is None:
             self._weighted_u = self.u*self.core
         return self._weighted_u
-    
+
     @property
     def weighted_v(self):
         '''
@@ -177,7 +177,7 @@ class SVD2DResults(object):
         for key, value in angles.iteritems():
             u_vector_magnitude = sqrt(u[key[0], :]*u[key[0], :])
             angles[key] = float(value)/(u_vector_magnitude*vec_magnitude)
-        
+
         return angles
 
     def weighted_u_vec(self, idx, default_zeros=False):
@@ -213,7 +213,7 @@ class SVD2DResults(object):
         for key, value in angles.iteritems():
             v_vector_magnitude = sqrt(v[key[0], :]*v[key[0], :])
             angles[key] = float(value)/(v_vector_magnitude*vec_magnitude)
-        
+
         return angles
 
     def weighted_v_vec(self, idx, default_zeros=False):
@@ -259,12 +259,22 @@ class SVD2DResults(object):
     def summarize_axis(self, axis, k=None, u_only=False, screen_width=80, output=sys.stdout):
         if k is None: k = len(self.svals)
         else: k = min(k, len(self.svals))
-        
-        print "\nAxis %d (sigma=%5.5f)" % (axis, self.svals[axis])
 
-        if u_only:
-            self.u[:,axis].show_extremes(10)
-            return
+        if isinstance(axis, int):
+            print "\nAxis %d (sigma=%5.5f)" % (axis, self.svals[axis])
+            if u_only:
+                self.u[:,axis].show_extremes(10)
+                return
+
+            u_slice, v_slice = self.u[:, axis], self.v[:, axis]
+
+        else:
+            print 'Ad-hoc axis (magnitude=%5.5f)' % axis.norm()
+            assert not u_only # not implemented yet.
+            u_slice = self.u_dotproducts_with(axis)
+            v_slice = self.v_dotproducts_with(axis)
+
+        u_extremes, v_extremes = u_slice.extremes(), v_slice.extremes()
 
         # Format in two columns
         if screen_width is not None:
@@ -281,7 +291,7 @@ class SVD2DResults(object):
         def utf8(s):
             if isinstance(s, unicode): return s.encode('utf-8')
             else: return str(s)
-            
+
         def fix_key(key):
             if not isinstance(key, tuple): return key
             if len(key) == 1: key = key[0]
@@ -296,9 +306,6 @@ class SVD2DResults(object):
                     key = u'%s/%s' % (rel, concept)
             return key
 
-        u_extremes = self.u[:, axis].extremes()
-        v_extremes = self.v[:, axis].extremes()
-
         for idx in xrange(20):
             if idx == 10: # halfway through
                 print >> output
@@ -311,7 +318,7 @@ class SVD2DResults(object):
             print >> output, '%s  %s' % (fit_to_width(left),
                                          fit_to_width(right))
 
-    
+
     def summarize(self, k=None, u_only=False, screen_width=80, output=sys.stdout):
         '''
         For each axis (up to the *k*-th axis), print the items of the
@@ -327,7 +334,7 @@ class SVD2DResults(object):
         Get the sigma values as an array of floats, no matter what they were before.
         '''
         return numpy.array([float(self.svals[i]) for i in range(len(self.svals))])
-    
+
     def export_svdview(self, outfn, packed=True, **kw):
         ''' Output a tab-separated values file suitable for use with
         svdview. The data is saved to the file named _outfn_. See the
@@ -381,13 +388,13 @@ class SVD2DResults(object):
         warn(DeprecationWarning('v_distances_to is named incorrectly; use v_dotproducts_with'))
         return self.v_dotproducts_with(vec)
 
-            
+
 class Reconstructed2DTensor(Tensor):
     '''
-    This class wraps the results of a 2-D SVD (represented by 
-    an :class:`SVD2DResults` object) and 
-    behaves like the :class:`Tensor` created 
-    by multiplying together the *U*, *S* and *V* matrices of the 
+    This class wraps the results of a 2-D SVD (represented by
+    an :class:`SVD2DResults` object) and
+    behaves like the :class:`Tensor` created
+    by multiplying together the *U*, *S* and *V* matrices of the
     SVD result.
 
     Reconstructed2DTensors require much less storage space than the
@@ -398,7 +405,7 @@ class Reconstructed2DTensor(Tensor):
     ndim = 2
     def __init__(self, svd):
         '''
-        The constructor creates a Reconstructed2DTensor from 
+        The constructor creates a Reconstructed2DTensor from
         a :class:`SVD2DResults` object.
         '''
         self.svd = svd
@@ -406,7 +413,7 @@ class Reconstructed2DTensor(Tensor):
 
     def __repr__(self):
         return u'<%r shape=%r svd=%r>' % (self.__class__, self.shape, self.svd)
-    
+
     def __len__(self):
         return self.shape[0] * self.shape[1]
 
@@ -418,10 +425,10 @@ class Reconstructed2DTensor(Tensor):
             raise DimensionMismatch
         i0, i1 = idx
 
-        if i0 == TheEmptySlice and i1 == TheEmptySlice: 
+        if i0 == TheEmptySlice and i1 == TheEmptySlice:
             return self
         elif i0 == TheEmptySlice:
-            return self.svd.weighted_u*self.svd.v[i1, :]            
+            return self.svd.weighted_u*self.svd.v[i1, :]
         elif i1 == TheEmptySlice:
             return self.svd.v*self.svd.weighted_u[i0, :]
         elif isinstance(i0, slice) or isinstance(i1, slice):
@@ -440,7 +447,7 @@ class Reconstructed2DTensor(Tensor):
             return self.svd.v.iter_dim_keys(0)
         else:
             raise DimensionMismatch
-    
+
     def sample(self, source, power=2):
         def sample_dist(vec):
             """
@@ -452,21 +459,21 @@ class Reconstructed2DTensor(Tensor):
             cumulprob = numpy.cumsum(no_neg) / numpy.sum(no_neg)
             index = numpy.searchsorted(cumulprob, random.random())
             return vec.label(0, index), no_neg[index]
-        
+
         while True:
             which_sval, prob1 = sample_dist(self.svd.svals)
             which_u, prob2 = sample_dist(self.svd.u[:, which_sval])
             which_v, prob3 = sample_dist(self.svd.v[:, which_sval])
-            
+
             # don't sample things we already knew
             if source[which_u, which_v] > 0: continue
-            if (isinstance(which_v, basestring) and 
+            if (isinstance(which_v, basestring) and
                 (which_v.startswith('InheritsFrom') or
                  which_v.endswith('InheritsFrom'))):
                 continue
             if (isinstance(which_v, tuple) and
                 (which_v[1] == 'InheritsFrom')): continue
-            
+
             u_modified = numpy.maximum(self.svd.u[which_u, :]._data, 0)
             v_modified = numpy.maximum(self.svd.v[which_v, :]._data, 0)
             u_modified2 = numpy.minimum(self.svd.u[which_u, :]._data, 0)
@@ -509,28 +516,28 @@ class DenormalizedReconstructed2DTensor(Reconstructed2DTensor):
             raise ValueError('Only the `:` slice supported.')
         else:
             return self.svd.u[i0, :]*self.svd.core*self.svd.v[i1, :] * self.norms[i0]
-        
-def svd_sparse(tensor, k=50):
+
+def svd_sparse(tensor, k=50, **kw):
     '''
     Compute the SVD of sparse tensor *tensor* using Lanczos' algorithm.
-    The optional parameter *k* is the number of sigma values to retain 
+    The optional parameter *k* is the number of sigma values to retain
     (default value is 50).
     '''
     if tensor.ndim < 2:
         raise NotImplementedError
     elif tensor.ndim == 2:
-        return svd_sparse_order2(tensor, k)
+        return svd_sparse_order2(tensor, k, **kw)
     else:
-        return svd_sparse_orderN(tensor, k)
+        return svd_sparse_orderN(tensor, k, **kw)
 
-def svd_sparse_order2(tensor, k=50):
+def svd_sparse_order2(tensor, k=50, **kw):
     '''
-    Compute the SVD of sparse 2-D tensor *tensor*. The optional 
-    parameter *k* is the number of sigma values to retain (default 
+    Compute the SVD of sparse 2-D tensor *tensor*. The optional
+    parameter *k* is the number of sigma values to retain (default
     value is 50).
     '''
 
-    ut, s, vt = tensor._svd(k=k)
+    ut, s, vt = tensor._svd(k=k, **kw)
     #cnorms = dMatNorms(vt)
 
 
@@ -545,7 +552,7 @@ def svd_sparse_order2(tensor, k=50):
 def svd_sparse_orderN(tensor, k=50):
     '''
     Compute the higher-order SVD of *tensor*. The optional
-    parameter *k* is the number of sigma values to retain (default 
+    parameter *k* is the number of sigma values to retain (default
     value is 50).
     '''
 
@@ -568,20 +575,20 @@ def svd_sparse_orderN(tensor, k=50):
 
 def incremental_svd(tensor, k=50, niter=100, lrate=.1, logging_interval=None):
     '''
-    Compute SVD of 2-D tensor *tensor* using an incremental SVD algorithm. 
-    This algorithm ignores unknown entries in the sparse tensor instead 
+    Compute SVD of 2-D tensor *tensor* using an incremental SVD algorithm.
+    This algorithm ignores unknown entries in the sparse tensor instead
     of treating them as zeros (like Lanczos' algorithm does). This means that
     the incremental SVD of a sparse matrix will not necessarily equal the
     Lanczos' SVD of the same matrix.
 
-    The incremental SVD algorithm performs gradient descent to find the 
+    The incremental SVD algorithm performs gradient descent to find the
     *U*, *V* and *S* matrices that minimize the
     Root Mean Squared error of the reconstructed matrix.
 
-    The optional parameter *k* is the number of sigma values to retain 
-    (default 50). The *lrate* parameter controls the "learning rate," a 
-    factor that changes the rate of gradient descent (default .1). A larger value 
-    means the gradient descent will take larger steps toward its goal. 
+    The optional parameter *k* is the number of sigma values to retain
+    (default 50). The *lrate* parameter controls the "learning rate," a
+    factor that changes the rate of gradient descent (default .1). A larger value
+    means the gradient descent will take larger steps toward its goal.
     Setting *lrate* to a "large" value may cause the algorithm to diverge,
     and will cause a mysterious overflow error.
     The *niter* parameter controls the number of steps performed by the
@@ -591,7 +598,6 @@ def incremental_svd(tensor, k=50, niter=100, lrate=.1, logging_interval=None):
     if not isinstance(tensor, Tensor):
         raise TypeError("This function can only be run on a Tensor. Use the "
                         ".incremental_svd() method for other objects.")
-        
+
     u, v, sigma = tensor._isvd(k=k, niter=niter, lrate=lrate)
     return SVD2DResults(u, v, sigma)
-    
