@@ -47,16 +47,16 @@ class Tensor(MyDictMixin):
     """
     Tensors are the main type of object handled by Divisi. This is the base
     class for all Tensors.
-        
+
     Tensors act like dictionaries whenever possible, so you can use
     dictionary methods such as ``keys``, ``iteritems``, and ``update`` as you
     would for a dictionary. They also act like Numpy arrays in some ways, but
     acting like a dictionary takes priority.
-    
+
     Many tensor operations refer to *modes*; these are the dimensions by which
     a tensor is indexed. For example, a matrix has two modes, numbered 0 and 1.
     Mode 0 refers to rows, and mode 1 refers to columns.
-    
+
     *Obscure terminology*: For higher-dimensional tensors, mode 2 has sometimes
     been called "tubes". Modes 3 and higher don't have names.
     """
@@ -66,7 +66,7 @@ class Tensor(MyDictMixin):
             return self.__iter__().next()
         except StopIteration:
             return None
-    
+
     @property
     def dims(self):
         """The number of dimensions of data in this tensor. For compatibility
@@ -78,13 +78,13 @@ class Tensor(MyDictMixin):
         Increment the value at key by amt.
         '''
         self[key] += amt
-        
+
     def __len__(self):
         """
         The number of *entries* stored in this tensor.
-        
+
         This does not count the implied zeros in a :class:`DictTensor`.
-        
+
         Note that this differs from the len() of a Numpy array, which would
         instead count its number of *rows*.
         """
@@ -94,9 +94,9 @@ class Tensor(MyDictMixin):
     def iter_dim_keys(self, mode):
         '''
         Get an iterator over the keys of a specified mode of the tensor.
-        
+
         Example usage::
-        
+
             for row in tensor.iter_dim_keys(0):
                 do_something_to(row)
         '''
@@ -112,13 +112,13 @@ class Tensor(MyDictMixin):
         return list(self.iter_dim_keys(mode))
 
     def combine_by_element(self, other, op):
-        '''  
+        '''
         Takes two tensors and combines them element by element using `op`.
-        
+
         For example, given input tensors *a* and *b*, ``result[i] = op(a[i],
         b[i])``, for all indices *i* in *a* and *b*. This operation requires *a*
         and *b* to have the same indices; otherwise, it doesn't make any sense.
-        
+
         (Note that, for the sake of efficiency, this doesn't run *op* on keys that
         neither *a* nor *b* have.)
         '''
@@ -133,7 +133,7 @@ class Tensor(MyDictMixin):
             seen_keys[key] = True
 
         # Run the operation for all items in the other tensor that we haven't already
-        # evaluated. 
+        # evaluated.
         for key, value in other.iteritems():
             if key not in seen_keys:
                 res[key] = op(self[key], other[key])
@@ -144,12 +144,12 @@ class Tensor(MyDictMixin):
     def slice(self, mode, index):
         """
         Gets a tensor of only the entries indexed by *index* on mode *mode*.
-        
+
         The resulting tensor will have one mode fewer than the original tensor.
         For example, a slice of a 2-D matrix is a 1-D vector.
-        
+
         Examples:
-        
+
         - To get a particular row of a matrix, use ``matrix.slice(0, row)``.
         - To get a particular column, use ``matrix.slice(1, col)``.
         """
@@ -160,19 +160,19 @@ class Tensor(MyDictMixin):
     def label(self, mode, index):
         """
         Returns the index as the *label* for that entry, so the label for an
-        unlabeled index is simply the index itself.  Added to allow certain 
+        unlabeled index is simply the index itself.  Added to allow certain
         operations which work on labeled views to work on all tensors.
         """
-        return index 
+        return index
 
     def add_layer(self, cls, *a, **kw):
         """
         Layer a view onto this tensor.
-        
+
         Tensors can be wrapped by various kinds of
         :class:`View`. This method adds a view in the
         appropriate place.
-        
+
         In this case, this is a plain Tensor, so it simply
         passes it to the view's constructor, but other Views will override
         this to layer on the new View in the appropriate way.
@@ -191,7 +191,7 @@ class Tensor(MyDictMixin):
     def labeled(self, labels):
         """
         Get a :class:`LabeledView` of this tensor.
-        
+
         *labels* should be a list of
         :class:`OrderedSet` s, one for each mode,
         which assign labels to its indices, or ``None`` if that mode should
@@ -204,9 +204,29 @@ class Tensor(MyDictMixin):
         """
         Get a :class:`divisi.normalized_view.NormalizedView`
         of this tensor.
+        
+        The parameter can be:
+         an int or tuple of ints: normalize along that dimension
+         True: normalize along axis 0
+         'tfidf': use tf-idf
+         'tfidf.T': use tf-idf, transposed (matrix is documents by terms)
+         a class: adds that class as a layer.
         """
-        from csc.divisi.normalized_view import NormalizedView
-        return self.add_layer(NormalizedView, mode)
+        if mode is True: mode = 0 # careful, 1 == True but, but 1 is not True.
+        if isinstance(mode, (int, tuple, list)):
+            from csc.divisi.normalized_view import NormalizedView
+            return self.add_layer(NormalizedView, mode)
+        elif mode == 'tfidf':
+            from csc.divisi.normalized_view import TfIdfView
+            return self.add_layer(TfIdfView)
+        elif mode == 'tfidf.T':
+            from csc.divisi.normalized_view import TfIdfView
+            return self.add_layer(TfIdfView, transposed=True)
+        elif isinstance(mode, type):
+            return self.add_layer(mode)
+
+        raise TypeError('Unknown normalization parameter %r' % mode)
+            
 
     def zero_mean_normalized(self, mode=0):
         from csc.divisi.normalized_view import ZeroMeanView
@@ -217,13 +237,23 @@ class Tensor(MyDictMixin):
         res = copy.deepcopy(self)
         res += other
         return res
-        
+
     def __iadd__(self, other):
         if isscalar(other):
             return self.scalar_iadd(other)
         else:
             return self.tensor_iadd(other)
 
+    def weighted_add(self, other, weight):
+        '''
+        this + (weight * other).
+        '''
+        return self + weight * other
+
+    def weighted_iadd(self, other, weight):
+        self += other * weight
+        return self
+
     def __sub__(self, other):
         return self + (-other)
 
@@ -234,7 +264,7 @@ class Tensor(MyDictMixin):
         """
         This performs matrix multiplication, not elementwise multiplication.
         See the documentation for :meth:`dot`.
-        
+
         If *other* is a scalar, this will perform scalar multiplication
         instead.
         """
@@ -267,24 +297,24 @@ class Tensor(MyDictMixin):
         The ``*=`` operator uses this method.
         '''
         raise NotImplementedError, "Don't know how to multiply by a constant in place for %s" % type(self)
-        
+
     def dot(self, other):
         """
         Get the product of two tensors, using matrix multiplication.
-        
+
         When two tensors *a* and *b* are multiplied, the entries in the result
         come from the dot products of the last mode of *a* with the first mode
         of *b*. So the product of a *k*-dimensional tensor with an
         *m*-dimensional tensor will have *(k + m - 2)* dimensions.
-        
+
         The ``*`` operation on two tensors uses this method.
         """
         raise NotImplementedError
-    
+
     def cmul(self, other):
         """
         Returns the product of this tensor and a scalar.
-        
+
         The ``*`` operation does this when given a scalar as its second argument.
         """
         raise NotImplementedError, "Don't know how to cmul a %s" % type(self)
@@ -292,7 +322,7 @@ class Tensor(MyDictMixin):
     def __div__(self, other):
         """
         Performs scalar division.
-        
+
         It actually just multiplies by 1/*other*, so if you're clever enough
         you might be able to make it divide by some other kind of object.
         """
@@ -301,7 +331,7 @@ class Tensor(MyDictMixin):
     def __idiv__(self, other):
         """
         Performs in-place scalar division.
-        
+
         It actually just multiplies by 1/*other*, so if you're clever enough
         you might be able to make it divide by some other kind of object.
         """
@@ -311,7 +341,7 @@ class Tensor(MyDictMixin):
     def norm(self):
         '''
         Calculate the Frobenius (or Euclidean) norm of this tensor.
-        
+
         The Frobenius norm of a tensor is simply the square root of the sum of
         the squares of its elements. For a vector, this is the same as the
         Euclidean norm.
@@ -324,7 +354,7 @@ class Tensor(MyDictMixin):
         return math.sqrt(mag)
 
     magnitude = norm
-    
+
     def hat(self):
         '''
         Return a normalized version of this tensor (generally a vector), by
@@ -335,16 +365,11 @@ class Tensor(MyDictMixin):
         NormalizedView.
         '''
         return self / max(self.norm(), EPSILON)
+    normalized_copy = hat
 
     ## Vector operations
     # Most of the operations below only make sense on vectors.
-    
-    def normalized_copy(self):
-        '''
-        Return a normalized copy of this tensor. For a vector, this is
-        the unit vector in the same direction.
-        '''
-        return self / self.norm()
+
 
 
     def projection_onto(self, other):
@@ -354,7 +379,7 @@ class Tensor(MyDictMixin):
         assert self.ndim == 1
         return (self * other) / (other * other) * other
 
-    
+
     def components_wrt(self, other):
         '''
         Return the (parallel, perpendicular) components of this vector
@@ -372,10 +397,10 @@ class Tensor(MyDictMixin):
         '''
         if self.ndim != 1: raise TypeError('Must be a vector')
         return (self*other) / (self.norm() * other.norm())
-    
+
 
     ## Summarization operations
-    def extreme_items(self, n_biggest=10, n_smallest=10):
+    def extreme_items(self, n_biggest=10, n_smallest=10, filter=None, detuple=False):
         '''
         Extract the items at either extreme of the tensor. Returns
         (biggest, smallest).
@@ -385,36 +410,64 @@ class Tensor(MyDictMixin):
         highest magnitude. *extremes* returns the *n_biggest* biggest
         and *n_smallest* smallest values in a tensor, along with their
         indices, in the form used by ``dict.items()``.
-        
-        If this is a vector (as it often is), note that the indices are going
-        to be tuples with one thing in them, for consistency.
 
-        TODO:
-        Somewhat like Python's ``sorted`` function, you can specify a
-        ``key``, e.g., abs, which is applied to the value before it is
-        compared. The output uses the original values.
+        The results are ordered by increasing value. For example, if
+        you ask for::
+
+            biggest, smallest = tensor.extreme_items(10, 10)
+
+        Then the smallest item will be found as ``smallest[0]`` and the
+        biggest as ``biggest[-1]``.
+
+        If this is a vector (as it often is), note that the indices
+        are going to be tuples with one thing in them, for
+        consistency. If you don't like that, pass detuple=True.
+
+        You can pass a ``filter(key, value)`` that will be called on
+        each item; the item will be included only if the filter
+        returns a true value.
         '''
+#         TODO:
+#         Somewhat like Python's ``sorted`` function, you can specify a
+#         ``key``, e.g., abs, which is applied to the value before it is
+#         compared. The output uses the original values.
+        if detuple and self.ndim != 1:
+            raise IndexError('Can only detuple for vectors.')
+        
+        sentinel = object()
+        
         from heapq import heapreplace
         from sys import maxint
-        biggest = [(-maxint,None)] * n_biggest
-        smallest = [(-maxint,None)] * n_smallest # maintains the negative of the smallest part
+        biggest = [(-maxint,sentinel)] * n_biggest
+        smallest = [(-maxint,sentinel)] * n_smallest # maintains the negative of the smallest part
 
-        # optimized case: no key
-        for k, v in self.iteritems():
+        # Filter
+        if filter is None:
+            items = self.iteritems()
+        else:
+            items = ((k, v) for k, v in self.iteritems() if filter(k, v))
+
+        # Pop the smallest values out, push larger values in. The heap
+        # makes this efficient.
+        for k, v in items:
             if n_biggest and v > biggest[0][0]: heapreplace(biggest, (v, k))
             if n_smallest and -v > smallest[0][0]: heapreplace(smallest, (-v, k))
-            
+
         biggest.sort()
         smallest.sort(reverse=True)
-        biggest = [(k, v) for v, k in biggest]
-        smallest = [(k, -v) for v, k in smallest]
+        biggest = [(k, v) for v, k in biggest if k is not sentinel]
+        smallest = [(k, -v) for v, k in smallest if k is not sentinel]
+
+        if detuple:
+            biggest = [(k, v) for (k,), v in biggest]
+            smallest = [(k, v) for (k,), v in smallest]
 
         return biggest, smallest
-        
+
     def extremes(self, n=10, top_only=False):
         """
         Extract the interesting parts of this tensor.
-        
+
         """
         n_biggest = n
         if top_only: n_smallest = 0
@@ -450,9 +503,9 @@ class Tensor(MyDictMixin):
         """
         return self._data
 
-    def svd(self, k=50):
+    def svd(self, k=50, *a, **kw):
         from csc.divisi.svd import svd_sparse
-        return svd_sparse(self, k)
+        return svd_sparse(self, k, *a, **kw)
 
     def _svd(self, k=50, *a, **kw):
         return svd(self, k=k, *a, **kw)
@@ -460,21 +513,85 @@ class Tensor(MyDictMixin):
     def _isvd(self, k=50, niter=100, lrate=.001, *a, **kw):
         return isvd(self, k=k, niter=niter, lrate=lrate, *a, **kw)
 
+    def weighted_slice_sum(self, mode, weights,
+                           constant_weight=None, ignore_missing=False):
+        '''
+        Computes the weighted sum of slices along `mode`. Weights are
+        specified as (key, weight).
+
+        This is a slight generalization of the concept of an ad hoc category.
+
+        You can also pass in just a list of keys as weights and set
+        constant_weight to a value (like 1.0) that you want to weight
+        everything by.
+
+        Some items may be missing. Pass ignore_missing=True, and
+        you'll get back a tuple: (weighted sum, missing items).
+        '''
+        # TODO: this can be done more efficiently by avoiding making copies
+        # when multiplying.
+
+        if constant_weight is not None:
+            if constant_weight is True: constant_weight = 1.0
+            weight_iter = ((item, constant_weight) for item in weights)
+        else:
+            weight_iter = iter(weights)
+
+        res = None
+        missing = []
+
+        # Add in other items.
+        for key, weight in weight_iter:
+            try:
+                slice = self.slice(mode, key)
+            except KeyError: # key missing
+                if not ignore_missing:
+                    raise KeyError(key)
+                missing.append((key, weight))
+            else:
+                if res is None: # first successful time
+                    res = slice * weight # the multiplication makes it a copy.
+                else:
+                    res.weighted_iadd(self.slice(mode, key), weight)
+
+        if ignore_missing:
+            return res, missing
+        else:
+            return res
+
+    def label_histograms(self):
+        '''
+        Counts the number and magnitude of items in each row / column
+        '''
+        import numpy
+        counts = [numpy.zeros(n, dtype=numpy.uint16) for n in self.shape]
+        magnitudes = [numpy.zeros(n) for n in self.shape]
+
+        for k, v in self.iteritems():
+            vv = v**2
+            for count, mag, key in izip(counts, magnitudes, k):
+                count[key] += 1
+                mag[key] += vv
+
+        return (map(DenseTensor, counts),
+                map(DenseTensor, magnitudes))
+
+
 class InvalidLayeringException(StandardError):
     pass
 
 forward_to_tensor = set([
-        'shape', 'ndim', 'has_key', 'default_value'])
+        'shape', 'ndim', 'has_key', 'default_value', 'density'])
 
 class View(Tensor):
-    """    
+    """
     A *view* is a wrapper around a tensor (always ``self.tensor``)
     that performs some operations (usually ``__getitem__`` and/or
     ``__setitem__``) differently.
-    
+
     For almost all purposes, it acts like a ``Tensor`` itself. Unknown
     methods are passed through to the underlying ``Tensor``.
-    
+
     This is just the abstract base class. Some useful ``View``s
     include :class:`csc.divisi.labeled_view.LabeledView` and
     :class:`csc.divisi.labeled_view.NormalizedView`.
@@ -533,33 +650,66 @@ class View(Tensor):
         return self.tensor
 
 
+class TransposedView(View):
+    '''
+    A simple view that just transposes its input.
+    '''
+    def __init__(self, tensor):
+        if tensor.ndim != 2:
+            raise TypeError('Can only transpose matrices.')
+        View.__init__(self, tensor)
+        r, c = tensor.shape
+        self.shape = c, r
+
+    def __getitem__(self, item):
+        r, c = item
+        return self.tensor[c, r]
+
+    def iteritems(self):
+        for (r, c), v in self.tensor.iteritems():
+            yield (c, r), v
+
+    def __iter__(self):
+        for r, c in self.tensor.iterkeys():
+            yield c, r
+
+    def __setitem__(self, key, value):
+        r, c = key
+        self.tensor[c, r] = value
+
+    def __delitem__(self, key):
+        r, c = key
+        del self.tensor[c, r]
+
+
+    
 class DictTensor(Tensor):
     '''
     A sparse tensor that stores data in nested dictionaries.
-    
+
     The first level of dictionaries specifies the rows, the second level
     specifies columns, and so on for higher modes. Therefore, slicing by
     rows is the easiest to do. Despite this, you can slice on any mode,
     possibly returning a :class:`divisi.sliced_view.SlicedView`
     for the sake of efficiency.
-    
+
     DictTensors can save a lot of memory, can efficiently provide input
     to a Lanczos :ref:`SVD`, and work well with
     :class:`divisi.labeled_view.LabeledView` objects. However, for some
     operations you may need to convert the DictTensor to a
     :class:`DenseTensor`.
     '''
-    
+
     # Indicates that not all possible keys will necessarily be iterated over.
     # NOTE: not used anywhere yet.
     is_sparse = True
-    
+
     def __init__(self, ndim, default_value=0.0):
         """
         Create a new, empty DictTensor with *ndim* dimensions.
-        
+
         Frequently, *ndim* is 2, creating a sparse matrix.
-        
+
         default_value is the value of all unspecified entries. An SVD will
         only work when *default_value*=0.0.
         """
@@ -589,7 +739,7 @@ class DictTensor(Tensor):
 
     def __getstate__(self):
         return dict(self.__dict__, version=1)
-    
+
     def __setstate__(self, state):
         version = state.pop('version', None)
         if version is None:
@@ -610,14 +760,14 @@ class DictTensor(Tensor):
             self.__dict__.update(state)
         else:
             raise TypeError('unsupported version of DictTensor: '+str(version))
-    
+
     def __len__(self):
         '''
         Compute the number of specified items.
 
         Note that some specified items may be zero.
         '''
-        # If speed is important, store the counter and 
+        # If speed is important, store the counter and
         # increment the count every time an item is added.
         def subcount(dct, to_go):
             if to_go == 0:
@@ -666,11 +816,11 @@ class DictTensor(Tensor):
         """
         A tuple representing the shape of this tensor.
         """
-        return tuple(self._shape)        
+        return tuple(self._shape)
 
     # Support negative reference semantics
     # TODO: it would be nice if this method could be eliminated by
-    # somehow making _dict_walk directly return a reference to the 
+    # somehow making _dict_walk directly return a reference to the
     # *value* in the final dictionary instead of the final dictionary.
     def _adjust_index(self, index, dim):
         if index < 0:
@@ -695,7 +845,7 @@ class DictTensor(Tensor):
                 cur[index] = {}
                 cur = cur[index]
         return cur
-        
+
     # Note: __contains__ is defined in terms of has_key by DictMixin
 
     def has_key(self, indices):
@@ -719,14 +869,14 @@ class DictTensor(Tensor):
 
         try:
             return self._dict_walk(indices)
-            
+
         except TypeError: # Dictionary raises this when you try and index into it with a slice
-            # Two kinds of slices: 
+            # Two kinds of slices:
             if not isinstance(indices[0], slice)\
                     and all(isinstance(index, slice) for index in indices[1:]):
                 # The optimal kind of slice is retrieving an entire
                 # first dimension, e.g. tensor[1, :]. This resolves
-                # to a dictionary lookup. 
+                # to a dictionary lookup.
 
                 # This is sort of a hack...
                 t = DictTensor(self.dims - 1)
@@ -735,7 +885,7 @@ class DictTensor(Tensor):
                 t.default_value = self.default_value
                 return t
             else:
-                # The other slice is a SlicedView 
+                # The other slice is a SlicedView
                 from csc.divisi.sliced_view import SlicedView
                 return SlicedView(self, indices)
 
@@ -801,7 +951,7 @@ class DictTensor(Tensor):
                     to_purge.append(k)
         for k in to_purge:
             del data[k]
-            
+
     def incremental_svd(self, *a, **kw):
         """
         Take the singular value decomposition of this tensor using an
@@ -924,7 +1074,7 @@ class DictTensor(Tensor):
         for k in self.iterkeys():
             self[k] += other
         return self
-        
+
     def tensor_iadd(self, other):
         '''
         Element-by-element tensor addition. For all keys *k* in this
@@ -937,7 +1087,7 @@ class DictTensor(Tensor):
             if v:
                 self[k] += v
         return self
-        
+
     def __sub__(self, other):
         '''
         Element-by-element tensor subtraction. For all keys *k* in this
@@ -947,7 +1097,7 @@ class DictTensor(Tensor):
         res = copy.deepcopy(self)
         res -= other
         return res
-        
+
     def __isub__(self, other):
         '''
         Element-by-element tensor subtraction. For all keys *k* in this
@@ -968,10 +1118,10 @@ class DictTensor(Tensor):
                     dct[k] *= other
             else:
                 for d in dct.itervalues():
-                    subiter(d, to_go-1) 
+                    subiter(d, to_go-1)
         subiter(self._data, self.ndim-1)
         return self
-    
+
     def __neg__(self):
         '''
         Return a new tensor whose values are the negation of this
@@ -1040,10 +1190,13 @@ class DenseTensor(Tensor):
     operations, since it allows you to use Numpy's optimized math
     libraries.
     '''
-    
+
     def __init__(self, data):
         """Create a DenseTensor from a numpy array."""
-        self._data = data
+        if isinstance(data, DenseTensor):
+            self._data = data._data
+        else:
+            self._data = data
         self.ndim = len(data.shape)
 
 
@@ -1082,7 +1235,7 @@ class DenseTensor(Tensor):
 
     def itervalues(self):
         return iter(numpy.reshape(self._data, (-1,)))
-    
+
     def iteritems(self):
         return izip(iter(self), self.itervalues())
 
@@ -1128,7 +1281,7 @@ class DenseTensor(Tensor):
         if self.ndim != 2:
             raise DimensionMismatch('Can only transpose a 2D tensor')
         return DenseTensor(self._data.T)
-    
+
     @property
     def T(self):
         return self.transpose()
@@ -1152,7 +1305,7 @@ class DenseTensor(Tensor):
 
     def array_op(self, op, *args, **kwargs):
         """Apply a Numpy operation to this tensor, returning a new DenseTensor."""
-        
+
         def extract_data(t):
             if isinstance(t, DenseTensor): return t.to_array()
             else: return t
@@ -1160,7 +1313,7 @@ class DenseTensor(Tensor):
         newargs = [self.to_array()] + [extract_data(a) for a in args]
         result = op(*newargs, **kwargs)
         return DenseTensor(result)
-    
+
     def concatenate(self, other):
         """
         Make a new DenseTensor containing all the rows in this DenseTensor,
@@ -1174,14 +1327,14 @@ class DenseTensor(Tensor):
 
     def k_means_inner(self, k, tolerance, iterlim):
         m, n = self.shape
-        
+
         # Normalize the data so that we're working with directions
         directions = self._data[:]
         for row in xrange(m):
             norm = math.sqrt(numpy.dot(directions[row], directions[row]))
             if norm == 0: continue
             directions[row] /= norm
-        
+
         # Initialize to random points chosen from the data
         unique = []
         for i in xrange(m):
@@ -1232,7 +1385,7 @@ class DenseTensor(Tensor):
             if iteration > iterlim:
                 logging.warn('k_means iteration limit exceeded')
                 break
-        
+
         clusterlists = [[] for i in xrange(k)]
         for row in xrange(m):
             cluster = clusters[row]
@@ -1240,6 +1393,12 @@ class DenseTensor(Tensor):
 
         return DenseTensor(means), clusterlists, dist
 
+    def _svd(self, k=50, *a, **kw):
+        from csc.divisi.svd import SVD2DResults
+        # perform a NumPy SVD on the data
+        u, sigma, vt = numpy.linalg.svd(data(self), full_matrices=False)
+        return u.T, sigma, vt
+
     def k_means(self, k=20, iter=10, tolerance=0.001, iterlim=1000):
         """
         Divide the data into clusters using k-means clustering.
