@@ -18,7 +18,7 @@ class DictTensorTest(unittest.TestCase):
                        [None, 2,    3   ],
                        [4,    None, None],
                        [None, 5,    None]]
-    
+
     def test_initial(self):
         self.assertEqual(len(self.tensor), 0)
         self.assertEqual(len(self.tensor.keys()), 0)
@@ -42,7 +42,7 @@ class DictTensorTest(unittest.TestCase):
 
     def test_slice(self):
         self.tensor.update(nones_removed(nested_list_to_dict(self.slice_testcase)))
-        
+
         # Test end conditions: start index
         # is included in slice, end index is not
         slice = self.tensor[1:3, 0:2]
@@ -50,7 +50,7 @@ class DictTensorTest(unittest.TestCase):
                           [[None, 2],
                            [4, None]])
 
-        # Test that slicing on some dims correctly 
+        # Test that slicing on some dims correctly
         # reduces the dimensionality of the tensor
         slice = self.tensor[3, :]
         assertTensorEqual(slice, [None, 5, None])
@@ -72,7 +72,7 @@ class DictTensorTest(unittest.TestCase):
                           [[1, None, 4, None],
                            [None, None, None, 5],
                            [None, 3, None, None]])
-        
+
     def test_delete(self):
         self.tensor.update(nones_removed(nested_list_to_dict(self.slice_testcase)))
         assertTensorEqual(self.tensor, self.slice_testcase)
@@ -95,10 +95,10 @@ class DictTensorTest(unittest.TestCase):
 
     def setUp(self):
         self.tensor = DictTensor(2)
-                
+
     def test_1D(self):
         tensor_1D = DictTensor(1)
-        tensor_1D[2] = 1 
+        tensor_1D[2] = 1
 
         assertTensorEqual(tensor_1D,
                                [None, None, 1])
@@ -145,7 +145,7 @@ class DictTensorTest(unittest.TestCase):
         assertTensorEqual(t1, [[1, 2], [3, 4]])
         t1 *= 2
         assertTensorEqual(t1, [[2, 4], [6, 8]])
-        
+
     def testICdiv(self):
         t1 = tensor_from_nested_list([[2, 4], [6, 8]])
         t1 /= 2
@@ -163,7 +163,7 @@ class DictTensorTest(unittest.TestCase):
         self.assertEqual(self.tensor.norm(), sqrt(26.0))
         self.assertEqual(self.tensor.magnitude(), sqrt(26.0))
 
-            
+
 from numpy import zeros
 class DenseTensorTest(unittest.TestCase):
     def setUp(self):
@@ -219,7 +219,7 @@ class LabeledDictTensorTest(unittest.TestCase):
         for row in ['x', 'y', 'z']:
             for col in ['a', 'b']:
                 self.tensor[row, col] = ord(row)*100+ord(col)
-        
+
         slice = self.tensor['x', :]
         self.assertEqual(slice.shape, (2,))
         self.assertEqual(slice['a'], ord('x')*100+ord('a'))
@@ -243,7 +243,7 @@ class LabeledDictTensorTest(unittest.TestCase):
 
         indices = self.tensor.indices(('a','1'))
         self.assertEqual(self.tensor.tensor[indices], 2)
-        
+
         self.assert_(self.tensor._label_dims_correct())
 
     def test_contains(self):
@@ -268,7 +268,7 @@ class LabeledDictTensorTest(unittest.TestCase):
 
         t3 = t1 + t2
         # FIXME: FINISH THIS!
-        
+
 normalize_testcase = [[1, None],
                       [3, 4]]
 
@@ -357,12 +357,43 @@ class TfIdfTest(unittest.TestCase):
         self.assertEqual(tfidf.num_documents, 10000000)
         self.assertEqual(tfidf.num_docs_that_contain_term[cow], 1000)
         self.assertAlmostEqual(tfidf.idf(cow), 9.21, 2)
-        
+
         # The TF-IDF score is the product of these quantities: 0.03 * 9.21 = 0.28.
         score = tfidf[cow, 0]
         self.assertEqual(len(getattr(score, 'shape', ())), 0)
         self.assertAlmostEqual(score, 0.28, 2)
 
+    def test_transposed(self):
+        '''Run the same testcase, but with the matrix transposed.'''
+        tensor = DictTensor(2)
+        # Consider a document containing 100 words wherein the word cow appears 3 times.
+        # [specifically, let there be a document where 'cow' appears 3 times
+        #  and 'moo' appears 97 times]
+        doc = 0
+        cow = 1
+        moo = 2
+        tensor[doc, cow] = 3
+        tensor[doc, moo] = 97
+        # Following the previously defined formulas, the term frequency (TF) for cow is then 0.03 (3 / 100).
+        tfidf = TfIdfView(tensor, transposed=True) # (can't create it earlier b/c it's read-only)
+        self.assertEqual(tfidf.counts_for_document[doc], 100)
+        self.assertAlmostEqual(tfidf.tf(cow, doc), 0.03)
+
+        # Now, assume we have 10 million documents and cow appears in one thousand of these.
+        #  [specifically, let 'cow' appear in documents 0 and 10,000,000-1000+1 till 10,000,000
+        for doc in xrange(10000000-1000+1,10000000):
+            tensor[doc, cow] = 1
+
+        # Then, the inverse document frequency is calculated as ln(10 000 000 / 1 000) = 9.21.
+        tfidf = TfIdfView(tensor, transposed=True) # (have to update after adding the other docs)
+        self.assertEqual(tfidf.num_documents, 10000000)
+        self.assertEqual(tfidf.num_docs_that_contain_term[cow], 1000)
+        self.assertAlmostEqual(tfidf.idf(cow), 9.21, 2)
+
+        # The TF-IDF score is the product of these quantities: 0.03 * 9.21 = 0.28.
+        score = tfidf[0, cow]
+        self.assertEqual(len(getattr(score, 'shape', ())), 0)
+        self.assertAlmostEqual(score, 0.28, 2)
 
 class UnfoldedSparseTensorTest(unittest.TestCase):
     def setUp(self):
@@ -424,7 +455,7 @@ class UnfoldedSetTests(unittest.TestCase):
             IdentitySet(10),               # 10 items
             OrderedSet(['a','b','c','d']), # 4 items
             ]
-    
+
     def test_index(self):
         uset = UnfoldedSet(self.sets)
         self.assertEqual(uset.index((2, 0, 'a')), 0)
@@ -442,7 +473,7 @@ class UnfoldedSetTests(unittest.TestCase):
     def test_len(self):
         uset = UnfoldedSet(self.sets)
         self.assertEqual(len(uset), 5*10*4)
-    
+
     def test_from_unfolding(self):
         u0 = UnfoldedSet.from_unfolding(0, self.sets)
         self.assertEqual(u0[0], (0, 'a'))
@@ -497,10 +528,10 @@ class TestMult(unittest.TestCase):
             # Make a random, randomly-shaped 3D tensor
             shape = random.sample(xrange(1,30), 3)
             tensor = DenseTensor(numpy.random.random(shape))
-            
+
             # Pick a random one of those dimensions
             dim = random.randrange(3)
-        
+
             # Make a random vector of that length
             vec = DenseTensor(numpy.random.random((shape[dim],)))
 
@@ -520,7 +551,7 @@ class NormalizeTest(unittest.TestCase):
     data = [[-5, 0, 0, 0],
             [0, 1, -2, 3],
             [100, 2, 0, 0]]
-    
+
     def setUp(self):
         self.tensor = DenseTensor(zeros((3, 4)))
         self.tensor.update(nested_list_to_dict(self.data))
@@ -560,13 +591,13 @@ class SVD2DTest(unittest.TestCase):
         self.assertEqual(len(self.incremental.svals), self.incremental.v.shape[1])
         self.assertEqual(self.incremental.v.shape[0], self.tensor.shape[1])
 
-        assertTensorEqual(self.incremental.u, 
+        assertTensorEqual(self.incremental.u,
                                [[0, 0, 1],
                                 [0, 1, 0],
                                 [0, 0, 0],
                                 [1, 0, 0]])
 
-        assertTensorEqual(self.incremental.v, 
+        assertTensorEqual(self.incremental.v,
                                [[0, 0, sqrt(.2)],
                                 [1, 0, 0],
                                 [0, 1, 0],
@@ -582,13 +613,13 @@ class SVD2DTest(unittest.TestCase):
         self.assertEqual(len(self.svals), self.v.shape[1])
         self.assertEqual(self.v.shape[0], self.tensor.shape[1])
 
-        assertTensorEqual(self.u, 
+        assertTensorEqual(self.u,
                                [[0, 0, 1],
                                 [0, -1, 0],
                                 [0, 0, 0],
                                 [-1, 0, 0]], abs=True)
 
-        assertTensorEqual(self.v, 
+        assertTensorEqual(self.v,
                                [[0, 0, sqrt(.2)],
                                 [-1, 0, 0],
                                 [0, -1, 0],
@@ -618,8 +649,8 @@ class SVD2DTest(unittest.TestCase):
 
         assertTensorEqual(self.v.T * self.v,
                                identity)
-    
-    def test_variance(self):        
+
+    def test_variance(self):
         # Assert that the SVD explained some of the variance.
         diff_k3 = self.tensor - self.svd.reconstructed
         tensor_mag = self.tensor.magnitude()
@@ -647,9 +678,9 @@ class NormalizedSVD2DTest(unittest.TestCase):
         self.assertEqual(len(self.svals), self.v.shape[1])
         self.assertEqual(self.v.shape[0], self.tensor.shape[1])
 
-        # Assert that the singular values are decreasing 
-        for i in range(1,len(self.svals)): 
-            self.assert_(self.svals[i] < self.svals[i-1]) 
+        # Assert that the singular values are decreasing
+        for i in range(1,len(self.svals)):
+            self.assert_(self.svals[i] < self.svals[i-1])
 
     def test_reconstructed(self):
         pass # TODO
@@ -657,8 +688,8 @@ class NormalizedSVD2DTest(unittest.TestCase):
     def test_orthonormality(self):
         assertTensorEqual(self.u.T * self.u, numpy.eye(self.u.shape[1]))
         assertTensorEqual(self.v.T * self.v, numpy.eye(self.u.shape[1]))
-    
-    def test_variance(self):        
+
+    def test_variance(self):
         return # TODO
         # Assert that the SVD explained some of the variance.
         diff_k3 = self.tensor - self.svd.reconstructed
@@ -683,13 +714,13 @@ def tensor_from_nested_list(data, tensor_cls=DictTensor):
     t = tensor_cls(2)
     t.update(nested_list_to_dict(data))
     return t
-    
+
 
 class NormalizedSVD2DTest2(unittest.TestCase):
     def assertItemsEqual(self, t1, t2):
         for key in t1.iterkeys():
             self.assertAlmostEqual(t1[key], t2[key])
-        
+
 
     def test_svd(self):
         '''Ensure SVDs of tensors match up after normalizing.'''
@@ -714,7 +745,7 @@ class NormalizedSVD2DTest2(unittest.TestCase):
         self.assertItemsEqual(svd3.u, svd4.u)
         self.assertItemsEqual(svd3.svals, svd4.svals)
         self.assertItemsEqual(svd3.v, svd4.v)
-        
+
 
 if __name__ == '__main__':
     unittest.main()
